package xmdv.graphics;



/***********************************************************

**  CopyRight (c) by Xmdv Group, CS Department of WPI, 2003

**  All Rights Reserved.

**

**  Author: Jing Yang

**  Date:   06/2003

***********************************************************/



import java.awt.*;

import java.awt.event.*;

import java.awt.geom.Point2D;

import java.lang.Math;

import xmdv.dataproc.*;



/**

 * Abstract class for displaying a dataset on a grCanvas.  

 * It provides the mouse and key interactions.

 *

 * @author  Jing Yang

 * @version v1.0

 */

public abstract class GeneralDisplay {

    // Variables declaration

    /** grCanvas on which to draw the display */

    protected grCanvas canvas = null;

    

    /** ColorManager to manager colors for this display */

    protected ColorManager colorManager = null;

    

    /** Dataset that provides data for this display */

    protected Dataset dataset = null;

    // End of variables declaration

    

    /** Creates a new instance of GeneralDisplay */

    public GeneralDisplay() {

    }

    

    /** Creates a new instance of GeneralDisplay 

     * @param canvas grCanvas to be set for this display

     * @param colorManager ColorManager to be set for this display

     * @param dataset Dataset to be set for this display

     */

    public GeneralDisplay(grCanvas canvas, ColorManager colorManager, Dataset dataset) {

        setCanvas(canvas);

        setColorManager(colorManager);

        setDataset(dataset);

    }

    

    /**

     * Sets grCanvas for this display

     * @param canvas grCanvas to be set for this display

     */

    public void setCanvas( grCanvas canvas ) {

        this.canvas = canvas;

    }

        

    /**

     * Sets ColorManager for this display

     * @param colorManager ColorManager to be set for this display

     */

    public void setColorManager(ColorManager colorManager){ 

        this.colorManager = colorManager;

    }

    

    /**

     * Sets Dataset for this display

     * @param dataset Dataset to be set for this display

     */

    public void setDataset(Dataset dataset) { 

        this.dataset = dataset;

        setParameters();

    }

    

    /** 

     * Cascades changes of dataset to the display

     * This function should be called after the dataset changes.

     */

    public void updateParameters() {

        setParameters();

    }

    

    /** 

     * Judges if a display is valid

     * @return flase if there is no canvas, colormanager, 

     * or dataset set for this display, or the dimensionality of the

     * dataset is lower than the minimum number of dimensions required 

     * for this display

     */  

    public boolean isDisplayValid()

    {

        if (canvas == null) {

            return false;

        }

        if (colorManager == null) {

            return false;

        }

        if (dataset == null ) {

            return false;

        }

        if (!isDimensionalityValid()) {

            return false;

        }    

        return true;

    }

    

    /** 

     * draws this display on the canvas

     */

    public void canvasDraw() {

        if (isDisplayValid()) {

            draw();

        } else {

            canvas.clear(colorManager.getColor("background"));

        }

    }

    

    /** 

     * Rewrites this function to draw some simple elements on the canvas without 

     * erasing the old picture

     * 

     */

    public void quickDraw() {

    }

    

    /** key press handler */

    public void canvasInputKeyPress(int key) {

        if ( isDisplayValid() ) {

            inputKeyPress(key);

        }

    }

    

    /** key release handler */

    public void canvasInputKeyRelease(int key) {

        if ( isDisplayValid() ) {

            inputKeyRelease(key);

        }

    }

    

    /** mouse motion handler */

    public void canvasInputMouseMotion(Point2D.Double pt)

    {

        if ( isDisplayValid() ) {

            inputMouseMotion(pt);

        }

    }

    

    /** mouse button press handler */

    public void canvasInputButtonPress(int button, Point2D.Double pt)

    {

        if ( isDisplayValid() ) {

            inputButtonPress(button, pt);

        }

    }

    

    /** mouse button release handler */

    public void canvasInputButtonRelease(int button, Point2D.Double pt){

        if ( isDisplayValid() ) {

            inputButtonRelease(button, pt);

        }

    }

    

   

    /**

     * Rewrites this function to judge if the dataset contains 

     * the minimum dimensionality required by your display.

     * This function must be rewritten by subclasses.

     * @reture true if there is enough dimensions, elsewise returns false

     */

    protected abstract boolean isDimensionalityValid();  

    

    /**

     * Rewrites this function to set your display parameters.

     * This function must be rewritten in subclasses.

     */

    protected abstract void setParameters();



    /**

     * Rewrites this function to draw your display on the canvas. 

     * This function must be rewritten in subclasses.

     */

    protected abstract void draw();



    /**

     * Rewrites this function to handle your key press event. 

     * This function must be rewritten in subclasses.

     * @param key the key that is pressed

     */

    protected abstract void inputKeyPress(int key);



    /**

     * Rewrites this function to handle your key release event. 

     * This function must be rewritten in subclasses.

     * @param key the key that is released

     */

    protected abstract void inputKeyRelease(int key);



    /**

     * Rewrites this function to handle your mouse motion event. 

     * This function must be rewritten in subclasses.

     * @param pt new mouse position

     */

    protected abstract void inputMouseMotion(Point2D.Double pt);



    /**

     * Rewrites this function to handle your mouse button press event. 

     * This function must be rewritten in subclasses.

     * @button mouse button that is pressed

     * @param pt mouse position

     */

    protected abstract void inputButtonPress(int button, Point2D.Double pt);



    /**

     * Rewrites this function to handle your mouse button release event. 

     * This function must be rewritten in subclasses.

     * @button mouse button that is released

     * @param pt mouse position

     */

    protected abstract void inputButtonRelease(int button, Point2D.Double pt);

}